'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mainVisitor = undefined;

var _utils = require('../utils');

var _replaceChildVisitor = require('./replaceChildVisitor');

var t = require("babel-types");

var mainVisitor = exports.mainVisitor = {
  CallExpression: function CallExpression(path) {
    var nodes = path.node.arguments.filter(_utils.isUnderscore);

    if (nodes.length > 0) {

      var ids = nodes.map(function (x) {
        return path.scope.generateUidIdentifier(_utils.T_UNDERSCORE);
      });

      var body = t.blockStatement([t.returnStatement(path.node)]);

      path.replaceWith(t.arrowFunctionExpression(ids, body));

      ids.forEach(function (id) {
        path.traverse(_replaceChildVisitor.replaceChildVisitor, { id: id });
      });

      path.stop();
    }
  },
  MemberExpression: function MemberExpression(path) {
    var object = path.node.object;


    if ((0, _utils.isUnderscore)(object)) {
      var id = (0, _utils.generateIdFromScope)(path.scope);

      var body = t.blockStatement([t.returnStatement(path.node)]);

      path.replaceWith(t.arrowFunctionExpression([id], body));

      (0, _utils.replaceChildsUnderscoreWithId)(path, id);
    }
  },
  BinaryExpression: function BinaryExpression(path) {
    var _path$node = path.node,
        left = _path$node.left,
        right = _path$node.right;

    var nodes = [left, right].filter(_utils.isUnderscore);

    if (nodes.length > 0) {
      var ids = nodes.map(function (x) {
        return (0, _utils.generateIdFromScope)(path.scope);
      });

      var body = t.blockStatement([t.returnStatement(path.node)]);

      path.replaceWith(t.arrowFunctionExpression(ids, body));

      ids.forEach(function (id) {
        return (0, _utils.replaceChildsUnderscoreWithId)(path, id);
      });
    }
  }
};